package jogl;

import java.awt.image.ImageFilter;
import java.awt.image.ShortLookupTable;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.vecmath.*;

import utils.FileUtils;

import com.sun.corba.se.spi.orbutil.fsm.InputImpl;

import miniMesh.MeshViewer;
import miniMesh.MiniMesh;

public class BatchUtil {
	final static String referenceMeshFile = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Input\\s905_geometry_normals_combined.ply";

	final static String dataDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\";
	
	final public static String inputDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Input\\";
	final public static String markersDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Markers\\";
	final public static String extendedMarkersDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\ExtendedMarkers\\";
	//final static String alignedMarkersDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\AlignedMarkers\\";
	final public static String alignedDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Aligned\\";
	final static String baseMeshDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\BaseMeshes\\";
	final static String detailsDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Details\\";
	final static String displacementImageDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\DisplacementImages\\";

	final static String remeshedNoDisplacementDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\RemeshedNoDisplacement\\";
	final static String remeshedDir = "F:\\Princeton\\MERL\\source\\Mesh Conversion\\Data\\Remeshed\\";

	final static String outputDir = dataDir + "Output\\";
	final static String meshVectorDir = dataDir + "MeshVectors\\";
	final static String eigenFaceDir = dataDir + "EigenFaces\\";
	final static String eigenStatsDir = dataDir + "EigenStats\\";
	final static String eigenShortStatsDir = dataDir + "EigenShortStats\\";
	final static String projectionOnEigenFaceDir = dataDir + "ProjectionsOnEigenfaces\\";
	final public static String connectivityFile = "Data\\Reference\\referenceConnectivity.con";
	final static String fromMerlDir = dataDir + "FromMerl\\";
	final static String randomFaceDir = dataDir + "RandomFaces\\";
	final static String randomStatsDir = dataDir + "RandomStats\\";
	final static String randomShortStatsDir = dataDir + "RandomShortStats\\";
	final static String notInSetProjDir = dataDir + "NotInSetProjections\\";
	final static String HBStatsDir = dataDir + "HeegerBergenStats\\";	
	final static String shortStatsDir = dataDir + "ShortStats\\";
	final static String synthesisIterDir = dataDir + "SynthesisIterations\\";
	
	public static Matrix4d getRigidTrans(String fromMarker, String toMarker){
		Matrix4d trans = null;
		try{
			Process process = Runtime.getRuntime().exec(new String[]{"F:\\Princeton\\MERL\\MeshAlignment\\Release\\MeshAlignment.exe",
					fromMarker,toMarker});
			
			InputStream istream = process.getInputStream();
			InputStream errorStream = process.getErrorStream();
			while(true){
				int nextChar = errorStream.read();
				//System.out.println("here 2");
				if(nextChar == -1)
					break;
				System.out.print((char)nextChar);
			}
			BufferedReader reader = new BufferedReader(new InputStreamReader(istream));
			trans = new Matrix4d();
			for(int i = 0; i < 4; i++){
			 	String line = reader.readLine();
			 	StringTokenizer tokenizer = new StringTokenizer(line);
			 	double matrixRow[] = new double[4];
			 	for(int j = 0; j < matrixRow.length; j++)
			 		matrixRow[j] = Double.parseDouble(tokenizer.nextToken());
			 	trans.setRow(i, matrixRow);
			}
			//System.out.println(trans);
		}
		catch(Exception e){
			e.printStackTrace();
		}		
		return trans;
		
	}		

	public static Image makeDisplacementImage(String fromFile, String toFile){
		Mesh fromMesh = new Mesh(fromFile);
		Mesh toMesh = new Mesh(toFile);
		double[] details = fromMesh.projectOn(toMesh);
		Raster raster = new Raster(Loader.imageSize, Loader.imageSize, 
				details);
		return raster.getImage();
	}
	
	public static void createReMeshes(){
		File baseDirFile = new File(baseMeshDir);
		File baseMeshFiles[] = baseDirFile.listFiles();
		int numMeshes = baseMeshFiles.length;
		String detailFiles[] = new String[numMeshes];
		for(int i = 0; i < numMeshes; i++){
			String meshFile = baseMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			detailFiles[i] = detailsDir + meshFile + "det";
			Mesh.readDetails(detailFiles[i]);
		}
		
		File[] existingOutputs = (new File(remeshedDir)).listFiles();
		List existingList = new ArrayList();
		for(int i = 0; i < existingOutputs.length; i++)
			existingList.add(existingOutputs[i]);
		File[] withHeldVectors = (new File(meshVectorDir + "WithHold")).listFiles(); 
		for(int i = 0; i < withHeldVectors.length; i++){
			String fileStr = withHeldVectors[i].getAbsolutePath();
			fileStr = FileUtils.changeExtension(fileStr, "ply");
			existingList.add(new File(fileStr));
		}
		
		for(int i = 0; i < numMeshes; i++){
			String meshFile = baseMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			File remeshedFile = new File(remeshedDir + meshFile + "ply");
			if(existingList.contains(remeshedFile))
				continue;
			
			System.gc();

			Mesh baseMesh = new Mesh(baseMeshFiles[i].getAbsolutePath());			

			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
			baseMesh.applyDetail(Mesh.details[i][0]);
			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
			baseMesh.applyDetail(Mesh.details[i][1]);

			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.applyDetail(Mesh.details[i][2]);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.applyDetail(Mesh.details[i][3]);
			
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.savePly(remeshedNoDisplacementDir + meshFile + "ply", true);
			
			Raster raster = new Raster(Loader.imageSize, Loader.imageSize
					/*, baseMesh*/, Mesh.details[i][Mesh.details[i].length - 1]);
			Image image = raster.getImage();
			String imageFile = displacementImageDir + meshFile + "img";			
			image.save(imageFile);

			
			baseMesh.applyDetail(Mesh.details[i][4]);
			baseMesh.savePly(remeshedFile.getAbsolutePath(), true);									
			baseMesh.saveAsVector(meshVectorDir + meshFile + "vec");
			if(i == 0)
				baseMesh.saveConnectivity(connectivityFile);
		}
	}

	/*
	public static void createDisplacementImages(){
		File baseDirFile = new File(baseMeshDir);
		File baseMeshFiles[] = baseDirFile.listFiles();
		int numMeshes = baseMeshFiles.length;
		String detailFiles[] = new String[numMeshes];
		for(int i = 0; i < numMeshes; i++){
			String meshFile = baseMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			detailFiles[i] = detailsDir + meshFile + "det";
			Mesh.readDetails(detailFiles[i]);
		}
		
		for(int i = 0; i < numMeshes; i++){
			System.gc();
			Mesh baseMesh = new Mesh(baseMeshFiles[i].getAbsolutePath());
			
			baseMesh.setBaseBoundary();
			//baseMesh.addPlaneCoords(1);		

			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);

			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);

			
			Raster raster = new Raster(Loader.imageSize, Loader.imageSize, baseMesh, Mesh.details[i][Mesh.details[i].length - 1]);
			Image image = raster.getImage();
			
			String meshFile = baseMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			String imageFile = displacementImageDir + meshFile + "img";
			
			image.save(imageFile);
		}
	}
	*/
	public static String changeExtension(String file, String ext){
		int periodPos = file.lastIndexOf(".");
		if(periodPos == -1)
			return file + "." + ext;
		return file.substring(0, periodPos + 1) + ext;
	}
	
	public static void adjustStats(String baseMeshFile, String imageFile, String statsFile, String outputFile){
		System.out.println("Adjusting: " + baseMeshFile + ", " + 
				imageFile + ", " + statsFile + " -> " + outputFile);
		Mesh baseMesh = new Mesh(baseMeshFile);
		Image img = new Image(imageFile);
		LocalICDF stats = new LocalICDF(statsFile);
		img.matchStatisticsOf(stats);
		baseMesh.applyImageDetail(img);
		baseMesh.saveAs(outputFile);
	}
	
	public static void adjustStatsUsingMean(String baseMeshFile, String imageFile, String targetStatsFile, String outputFile){
		System.out.println("Adjusting: " + baseMeshFile + ", " + 
				imageFile + ", " + targetStatsFile + " -> " + outputFile);
		LocalShortStats stats = new LocalShortStats(targetStatsFile);
		adjustStats(baseMeshFile, imageFile, stats, outputFile);
		/*
		Mesh baseMesh = new Mesh(baseMeshFile);
		Image img = new Image(imageFile);
		LocalShortStats stats = new LocalShortStats(targetStatsFile);
		img.matchStatisticsOf(stats);
		baseMesh.applyImageDetail(img);
		baseMesh.saveAs(outputFile);
		*/
	}

	public static void adjustStats(String baseMeshFile, String imageFile, LocalStats targetStats, String outputFile){
		Mesh baseMesh = new Mesh(baseMeshFile);
		Image img = new Image(imageFile);
		img.matchStatisticsOf(targetStats);
		baseMesh.applyImageDetail(img);
		baseMesh.saveAs(outputFile);				
	}

	
	public static void adjustStatsDir(String statsDir, String baseMeshFile, String startImageFile){
		File plyDir = new File(statsDir + "ply");
		if(!plyDir.exists())
			plyDir.mkdir();
		

		File[] dirs = (new File(statsDir)).listFiles();
		for(int i = 0; i < dirs.length; i++){
			if(dirs[i].isDirectory() && !dirs[i].getName().equals("ply")){
				File[] statFiles = dirs[i].listFiles();
				String destinationDir = statsDir + "ply\\" + dirs[i].getName() + "\\";
				System.out.println(" -> " + destinationDir);
				File destinationDirF = new File(destinationDir);
				if(!destinationDirF.exists())
					destinationDirF.mkdir();
				for(int j = 0; j < statFiles.length; j++){
					String destinationFile = destinationDir + statFiles[j].getName();
					destinationFile = changeExtension(destinationFile, "ply");
					adjustStatsUsingMean(baseMeshFile, startImageFile, 
							statFiles[j].getAbsolutePath(), destinationFile);
				}
			}
		}
	}
	
	public static void createHBStats(){
		File inputDirFile = new File(displacementImageDir);
		File inputImageFiles[] = inputDirFile.listFiles();		
		int numImage = inputImageFiles.length;
		
		int numOrientations = 4, numScales = 4;
		
		System.out.println("Saving filter responses");
		String filterResponseDir = displacementImageDir + "FilterResponses\\";
		File newDirFile = new File(filterResponseDir);
		if(!newDirFile.exists())
			newDirFile.mkdir();
		for(int i = 0; i < inputImageFiles.length; i++){
			if(inputImageFiles[i].isDirectory())
				continue;
			String imageDir = filterResponseDir + FileUtils.getName(inputImageFiles[i].getName()) + "\\";
			File imageDirFile = new File(imageDir);
			if(imageDirFile.exists())
				continue;
			else
				imageDirFile.mkdir();
			
			Image image = new Image(inputImageFiles[i].getAbsolutePath());
			image.createSteerablePyramid(numOrientations, numScales);
			int numChannels = image.pyramid.bands.length;
			for(int j = 0; j < image.pyramid.bands.length; j++){
				String numString = String.valueOf(j);
				if(numString.length() == 1)
					numString = "0" + numString; 
				image.pyramid.bands[j].save(imageDir + numString + ".img");					
			}
		}
		
		File[] existingOutputs = (new File(HBStatsDir)).listFiles();
		List existingList = new ArrayList();
		for(int i = 0; i < existingOutputs.length; i++)
			existingList.add(existingOutputs[i]);

		System.out.println("Saving HB stats");
		
		for(int i = 0 ; i < inputImageFiles.length; i++){
			System.gc();
			if(inputImageFiles[i].isDirectory())
				continue;
			String HBFile = inputImageFiles[i].getName();
			HBFile = HBFile.substring(0, HBFile.length() - 3) + "hb";
			HBFile = HBStatsDir + HBFile;
			File overwriteFileHB = new File(HBFile);
			if(existingList.contains(overwriteFileHB))
				continue;

			Image image = new Image(inputImageFiles[i].getAbsolutePath());
			LocalICDF stats = new LocalICDF(image, 4, ImageLoader.numStatisticTiles);
			stats.saveAs(HBFile);
		}

		existingOutputs = (new File(shortStatsDir)).listFiles();
		existingList = new ArrayList();
		for(int i = 0; i < existingOutputs.length; i++)
			existingList.add(existingOutputs[i]);

		System.out.println("Saving short stats");

		for(int i = 0 ; i < inputImageFiles.length; i++){
			System.gc();
			if(inputImageFiles[i].isDirectory())
				continue;
			String simpleStatFile = inputImageFiles[i].getName();
			simpleStatFile = simpleStatFile.substring(0, simpleStatFile.length() - 3) + "ss";
			simpleStatFile = shortStatsDir + simpleStatFile;
			File overwriteFileSimple = new File(simpleStatFile);
			if(existingList.contains(overwriteFileSimple))
				continue;

			Image image = new Image(inputImageFiles[i].getAbsolutePath());
			LocalShortStats stats = new LocalShortStats(image, 4);
			stats.saveAs(simpleStatFile);
		}
	}		

	
	public static void createDetails(){
		File inputDirFile = new File(alignedDir);
		File inputMeshFiles[] = inputDirFile.listFiles();		
		List inputMeshes = new ArrayList();
		for(int i = 0; i < inputMeshFiles.length; i++)
			if(inputMeshFiles[i].isFile())
				inputMeshes.add(inputMeshFiles[i]);
		inputMeshFiles = new File[inputMeshes.size()];
		for(int i = 0; i < inputMeshFiles.length; i++)
			inputMeshFiles[i] = (File)inputMeshes.get(i);
		
		int numMeshes = inputMeshFiles.length;
		String markerFiles[] = new String[numMeshes];
		for(int i = 0; i < numMeshes; i++){
			String meshFile = inputMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			markerFiles[i] = extendedMarkersDir + meshFile + "mkr";
			System.out.println(markerFiles[i]);
		}		
		
		File[] existingOutputs = (new File(detailsDir)).listFiles();
		List existingList = new ArrayList();
		for(int i = 0; i < existingOutputs.length; i++)
			existingList.add(existingOutputs[i]);

		
		for(int i = 0 ; i < inputMeshFiles.length; i++){
			System.gc();

			String detailFile = inputMeshFiles[i].getName();
			detailFile = detailFile.substring(0, detailFile.length() - 3) + "det";
			detailFile = detailsDir + detailFile;
			File overwriteFile = new File(detailFile);
			if(existingList.contains(overwriteFile))
				continue;
			
			Mesh mesh = new Mesh(inputMeshFiles[i].getAbsolutePath());
			mesh.loadMarkers(markerFiles[i]);
			Mesh baseMesh = mesh.baseMesh();
			baseMesh.savePly(baseMeshDir + inputMeshFiles[i].getName(), true);

			try{
				FileOutputStream fstream = new FileOutputStream(detailFile);
				fstream.close();
			}
			catch(IOException e){
				e.printStackTrace();
			}
			
			
			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
			Loader.projectMeshOnMesh(baseMesh, mesh, detailFile);
			baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
			Loader.projectMeshOnMesh(baseMesh, mesh, detailFile);

			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			Loader.projectMeshOnMesh(baseMesh, mesh, detailFile);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			Loader.projectMeshOnMesh(baseMesh, mesh, detailFile);
			
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
			Loader.projectMeshOnMesh(baseMesh, mesh, detailFile);
			mesh = null;
		}
	}		
	
	public static void alignMeshes(){
		File inputDirFile = new File(inputDir);				
		File inputMeshFiles[] = inputDirFile.listFiles();
		if(referenceMeshFile != null){
			File refMeshFile = new File(referenceMeshFile);
			for(int i = 0; i < inputMeshFiles.length; i++){
				if(refMeshFile.equals(inputMeshFiles[i])){
					System.out.println("Found ref file: " + inputMeshFiles[i].getName());
					File temp = inputMeshFiles[0];
					inputMeshFiles[0] = inputMeshFiles[i];
					inputMeshFiles[i] = temp;
					break;
				}				
			}
		}
		
		int numMeshes = inputMeshFiles.length;
		String markerFiles[] = new String[numMeshes];
		for(int i = 0; i < numMeshes; i++){
			String meshFile = inputMeshFiles[i].getName();
			meshFile = meshFile.substring(0, meshFile.length() - 3);
			markerFiles[i] = markersDir + meshFile + "mkr";
			System.out.println(markerFiles[i]);
		}
		
		Mesh refMesh = new Mesh(inputMeshFiles[0].getAbsolutePath());		
		refMesh.savePly(alignedDir + inputMeshFiles[0].getName(), true);
		refMesh = null;
		
		File[] existingOutputs = (new File(alignedDir)).listFiles();
		List existingList = new ArrayList();
		for(int i = 0; i < existingOutputs.length; i++)
			existingList.add(existingOutputs[i]);
		
		for(int i = 1 ; i < inputMeshFiles.length; i++){						
			File outputFile = new File(alignedDir + inputMeshFiles[i].getName());
			if(existingList.contains(outputFile))
				continue;
			
			System.out.println("Before garbage collect");
			System.gc();
			System.out.println("after garbage collect");
			Matrix4d trans = getRigidTrans(markerFiles[i], markerFiles[0]);
			System.out.println("after transformation");
			Mesh mesh = new Mesh(inputMeshFiles[i].getAbsolutePath());
			Point3d pos = new Point3d(); 
			for(int j = 0; j < mesh.vertices.size(); j++){
				Vertex v = (Vertex)mesh.vertices.get(j);
				pos.set(v.pos);
				trans.transform(pos);
				v.pos.set(pos);			
			}
			
			mesh.savePly(outputFile.getAbsolutePath(), true);
		}
		
	}
		
	public static Mesh getAvgMesh(File[] meshFiles){
		Mesh mesh = new Mesh(meshFiles[0].getAbsolutePath());
		for(int i = 1; i < meshFiles.length; i++){
			System.gc();
			Mesh nextMesh = new Mesh(meshFiles[i].getAbsolutePath());
			if(nextMesh.vertices.size() != mesh.vertices.size())
				throw new RuntimeException();
			for(int j = 0; j < mesh.vertices.size(); j++){
				Vertex v1 = (Vertex)mesh.vertices.get(j);
				Vertex v2 = (Vertex)nextMesh.vertices.get(j);
				v1.pos.add(v2.pos);
			}
		}
		for(int j = 0; j < mesh.vertices.size(); j++){
			Vertex v = (Vertex)mesh.vertices.get(j);
			v.pos.scale(1.0 / meshFiles.length);
		}
		return mesh;
	}
		
	
	public static void ply2vec(String dirStr){
		File dir = new File(dirStr);
		File files[] = dir.listFiles();
		for(int i = 0; i < files.length; i++){
			if(!files[i].isDirectory()){
				MiniMesh mesh = new MiniMesh(files[i].getAbsolutePath());
				File vecDir = new File(dirStr + "vec\\");
				if(!vecDir.exists())
					vecDir.mkdir();

				String fileStr = files[i].getName();
				fileStr = fileStr.substring(0, fileStr.length() - 3);
				mesh.saveAs(vecDir.getAbsolutePath() + "\\" + fileStr + "vec");
			}
		}
	}

	public static void img2ply(String dirStr, String baseMeshFile){
		File dir = new File(dirStr);
		File files[] = dir.listFiles();
		Mesh baseMesh = new Mesh(baseMeshFile);
		for(int i = 0; i < files.length; i++){
			if(!files[i].isDirectory()){
				System.gc();
				File plyDir = new File(dirStr + "ply\\");
				if(!plyDir.exists())
					plyDir.mkdir();

				Mesh baseMesh2 = new Mesh(baseMesh); 
				baseMesh2.applyImageDetail(files[i].getAbsolutePath());
				
				String fileStr = files[i].getName();
				fileStr = fileStr.substring(0, fileStr.length() - 3);
				baseMesh2.saveAs(plyDir.getAbsolutePath() + "\\" + fileStr + "ply");
			}
		}
	}

	
	public static void mesh2bmp(String dirStr, int maxFiles, boolean center){
		int fileCounter = 0;
		File dir = new File(dirStr);
		File files[] = dir.listFiles();
		for(int i = 0; i < files.length; i++){
			if(!files[i].isDirectory()){
				System.gc();
				String ext = FileUtils.getExtension(files[i].getName());
				if(!ext.equals("ply") && !ext.equals("vec"))
					continue;
				fileCounter++;
				File bmpDir = new File(dirStr + "bmp\\");
				if(!bmpDir.exists())
					bmpDir.mkdir();

				String fileStr = files[i].getName();
				MeshViewer.takeScreenShot(files[i].getAbsolutePath(), 
						bmpDir.getAbsolutePath() + "\\" + 
						FileUtils.changeExtension(fileStr, "png"),
						true);
				
				if(fileCounter >= maxFiles)
					return;
				System.out.println();
			}
		}
	}

	public static void img2bmp(String dirStr, int maxFiles){
		int fileCounter = 0;
		File dir = new File(dirStr);
		File files[] = dir.listFiles();
		for(int i = 0; i < files.length; i++){
			if(!files[i].isDirectory()){
				System.gc();
				String ext = FileUtils.getExtension(files[i].getName());
				if(!ext.equals("img"))
					continue;
				fileCounter++;
				File bmpDir = new File(dirStr + "bmp\\");
				if(!bmpDir.exists())
					bmpDir.mkdir();

				String fileStr = files[i].getName();
				Image image = new Image(files[i].getAbsolutePath());
				image.saveAs(bmpDir.getAbsolutePath() + "\\" +
						FileUtils.changeExtension(fileStr, "bmp"), true);
				
				if(fileCounter >= maxFiles)
					return;
				System.out.println();
			}
		}
	}
	
	public static void vec2ply(String dirStr){
		vec2ply(dirStr, Integer.MAX_VALUE, Integer.MAX_VALUE);
	}
	public static void vec2ply(String dirStr, int maxFiles, int maxDirs){
		int fileCounter = 0;
		int dirCounter = 0;
		File dir = new File(dirStr);
		File files[] = dir.listFiles();
		for(int i = 0; i < files.length; i++){
			if(files[i].isDirectory()){
				dirCounter++;
				if(dirCounter > maxDirs)
					continue;
				vec2ply(files[i].getAbsolutePath(), maxFiles, maxDirs);
			}
			else{
				if(dirStr.charAt(dirStr.length() - 1) != '\\')
					dirStr = dirStr + "\\";
				File plyDir = new File(dirStr + "ply\\");
				String fileStr = files[i].getName();
				if(Mesh.getExtension(fileStr).equals("vec")){
					fileCounter++;
					if(fileCounter > maxFiles)
						continue;
					if(!plyDir.exists())
						plyDir.mkdir();
					System.gc();
					MiniMesh mesh = new MiniMesh(files[i].getAbsolutePath());
					fileStr = fileStr.substring(0, fileStr.length() - 3);
					//System.out.println(plyDir.getAbsolutePath() + fileStr + "ply");
					mesh.saveAs(plyDir.getAbsolutePath() + "\\" + fileStr + "ply");
				}
			}
		}
	}
	
	static void testMakeDispImage(){
		Image testImage = makeDisplacementImage(projectionOnEigenFaceDir + "s042-041019-02-optimized\\proj03.vec",
					alignedDir + "s042-041019-02-optimized.ply");
		testImage.save(dataDir + "testImage.img");
	}
	
	static void testSaveHGStats(){
		String imgFile = displacementImageDir + "s042-041019-02-optimized.img";
		LocalICDF first = new LocalICDF(new Image(imgFile), 4, ImageLoader.numStatisticTiles);
		String savedFile = "test.hb"; 
		first.saveAs(savedFile);
		
		LocalICDF second = new LocalICDF(savedFile);
		System.out.println(second.equals(first));
	}
	
	static void testAdjustStats(){
		adjustStats(outputDir + "avgBase.vec",
				outputDir + "meanImage.img",
				outputDir + "meanStat.hb",
				outputDir + "meanFaceAdjusted2.vec");
	}
	
	public static void addFilesToInput(){
		try{
			List processedNames = new ArrayList();
			File[] processedFiles = (new File(inputDir)).listFiles();
			for(int i = 0; i < processedFiles.length; i++)
				processedNames.add(processedFiles[i].getName());
			
			File[] dirs = (new File(fromMerlDir)).listFiles();
			for(int i = 0; i < dirs.length; i++){
				String name = dirs[i].getName();
				if(name.equals("processed"))
					continue;
				String targetFileName = name.substring(0, 4) + ".ply";
				System.out.println("Processing " + targetFileName);
				if(!processedNames.contains(targetFileName)){
					File[] files = dirs[i].listFiles();
					boolean foundPly = false;
					for(int j = 0; j < files.length; j++){
						if(FileUtils.getExtension(files[j].getName()).equals("ply")){
							foundPly = true;
							String fileName = files[j].getName();
							if(!processedNames.contains(fileName)){
								String copyFrom = "\"" + files[j].getAbsolutePath() + "\"";
								String copyTo = "\"" + inputDir + targetFileName + "\""; 
								//String copyCommand = "copy " + copyFrom + " " + copyTo;
								//System.out.println("  "  + copyCommand);
								Runtime.getRuntime().exec("cmd /c copy " + copyFrom + " " + copyTo);
								System.out.println("  copied " + targetFileName);
								//Runtime.getRuntime().exec(new String[]{"cmd /c copy"/*, copyFrom, copyTo*/});
							}
							else{
								System.out.println("  already exists: " + targetFileName);
							}
						}						
					}
					if(!foundPly)
						System.out.println("not found ply!!!! " + targetFileName);
				}
				else{
					System.out.println("  already exists: " + targetFileName);
				}
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	static void fixMarkerFile(String fromFile, String toFile){
		try{
			System.out.println("Fixing: " + fromFile + "  ->  " + toFile);
			BufferedReader reader = new BufferedReader(new FileReader(fromFile));
			String line = reader.readLine();
			int numMarkers = Integer.parseInt(line);
			System.out.println(fromFile + ": " + numMarkers);
			String lines[] = new String[numMarkers];
			line = reader.readLine();
			StringTokenizer tokenizer = new StringTokenizer(line);
			LinkedList tokens = new LinkedList();
			while(tokenizer.hasMoreTokens())
				tokens.add(tokenizer.nextToken());
			
			for(int i = 0; i < numMarkers; i++){
				String xStr = (String)tokens.getFirst();
				tokens.removeFirst();
				String yStr = (String)tokens.getFirst();
				tokens.removeFirst();
				String zStr = (String)tokens.getFirst();
				tokens.removeFirst();
				
				String indexStr = (String)tokens.getFirst();
				tokens.removeFirst();
				
				int indexMinus = indexStr.indexOf('-');
				if(indexMinus != -1){
					String nextStr = indexStr.substring(indexMinus, indexStr.length());
					tokens.addFirst(nextStr);
					indexStr = indexStr.substring(0, indexMinus);
				}
				else{
					int indexDot = indexStr.indexOf('.');
					if(indexDot != -1){
						indexDot--;
						String nextStr = indexStr.substring(indexDot, indexStr.length());
						tokens.addFirst(nextStr);
						indexStr = indexStr.substring(0, indexDot);
					}
				}
				
				lines[i] = xStr + " " + yStr + " " + zStr + " " + indexStr + "\n";
			}
			reader.close();
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(toFile)); 
			writer.write(numMarkers + "\n");
			for(int i = 0; i < numMarkers; i++)
				writer.write(lines[i]);
			writer.close();
		}
		catch(IOException e){
			System.out.println(e);
		}		

	}
	
	static void fixMarkerFiles(){
		File[] files = (new File(markersDir)).listFiles();
		for(int i = 0; i < files.length; i++){
			String fileName = files[i].getName();
			if(fileName.length() == 8){
				fixMarkerFile(files[i].getAbsolutePath(), files[i].getAbsolutePath());
			}
		}
	}
	
	static void removeFile(String subject){
		ArrayList filesToDelete = new ArrayList();
		//filesToDelete.add(new File(markersDir + subject + ".mkr"));
		//filesToDelete.add(new File(extendedMarkersDir + subject + ".mkr"));
		//filesToDelete.add(new File(alignedDir + subject + ".ply"));
		filesToDelete.add(new File(detailsDir + subject + ".det"));
		filesToDelete.add(new File(baseMeshDir + subject + ".ply"));
		filesToDelete.add(new File(displacementImageDir + subject + ".img"));
		File filterImagesDir = new File(displacementImageDir + "FilterResponses\\" + subject);
		File[] filterImages = filterImagesDir.listFiles(); 
		for(int i = 0; i < filterImages.length; i++)
			filesToDelete.add(filterImages[i]);
		filesToDelete.add(new File(HBStatsDir + subject + ".hb"));
		filesToDelete.add(new File(meshVectorDir + subject + ".vec"));
		filesToDelete.add(new File(remeshedDir + subject + ".ply"));
		filesToDelete.add(new File(remeshedNoDisplacementDir + subject + ".ply"));
		filesToDelete.add(new File(shortStatsDir + subject + ".ss"));
		
		for(int i = 0; i < filesToDelete.size(); i++)
			System.out.println("Deleting: " + filesToDelete.get(i) + "; " +
					((File)filesToDelete.get(i)).delete());
		
		System.out.println(filterImagesDir + ": " + filterImagesDir.delete());
	}
	
	static void adjust(String from, String to, String saveDir){
		LocalShortStats targetStats = new LocalShortStats(shortStatsDir + to + ".ss");
		Image image =  new Image(displacementImageDir + from + ".img");
		image.matchStatisticsOf(targetStats);
		image.save(saveDir + from + "_" + "to_" + to + ".img");		
		
		Mesh baseMesh = new Mesh(remeshedNoDisplacementDir + from + ".ply");
		baseMesh.applyImageDetail(image);
		baseMesh.saveAs(saveDir + from + "_" + "to_" + to + ".ply");

	}
	
	static void adjustLowRes(String from, String to, 
			String workingDir, String imageFile){
		adjustLowRes(from, to, workingDir, new Image(imageFile));
	}
		
	static void adjustLowRes(String from, String to, 
			String workingDir, Image image){
		System.gc();
		Mesh baseMesh = new Mesh(baseMeshDir + from + ".ply");

		Mesh.details = new double[0][][]; 

		Mesh.readDetails(detailsDir + from + ".det");
		Mesh.readDetails(detailsDir + to + ".det");
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][0]);
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][1]);

		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][2]);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][3]);
		
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);

		baseMesh.applyImageDetail(image);
		baseMesh.saveAs(workingDir + FileUtils.getName(image.fileName) + "_det0111" + ".ply");
	}
	
	
	static void aging(){
		//adjust("s133", "s060", outputDir + "aging\\");
		//adjust("s060", "s133", outputDir + "aging\\");
		/*
		adjust("s103", "s060", outputDir + "aging\\");
		adjust("s142", "s060", outputDir + "aging\\");
		adjust("s060", "s103", outputDir + "aging\\");
		adjust("s060", "s142", outputDir + "aging\\");
		*/
		//adjustLowRes("s133", "s060", outputDir + "aging\\");
		//adjustLowRes("s060", "s133", outputDir + "aging\\");
		
		//adjustLowRes("s905_lowRes4", "s060", outputDir + "aging\\");
		/*
		adjustLowRes("s103", "s060", outputDir + "aging\\");
		adjustLowRes("s142", "s060", outputDir + "aging\\");
		adjustLowRes("s060", "s103", outputDir + "aging\\");
		adjustLowRes("s060", "s142", outputDir + "aging\\");
		*/

	}
	
	static void testStatAdjustment(){
		/*
		adjustStats(outputDir + "subject-30k_base.ply", 
				outputDir + "temp\\subject-30k.img", 
				outputDir + "temp\\s60.hb",
				outputDir + "temp\\old_standard_synthesis.ply");
		*/
		
		/*
		adjustStatsUsingMean(outputDir + "subject-30k_base.ply", 
				outputDir + "temp\\subject-30k.img", 
				outputDir + "temp\\s60_geometry_normals_combined.ss",
				outputDir + "temp\\old_synthesis_using_mean2.ply");
		*/
		
		/*
		adjustStatsUsingMean(outputDir + "temp\\s905_base.ply", 
				outputDir + "temp\\s905_geometry_normals_combined.img", 
				outputDir + "temp\\s905_geometry_normals_combined.ss",
				outputDir + "temp\\resynthesis_using_mean.ply");
        */
		
		/*
		adjustStats(outputDir + "temp\\s905_base.ply", 
				outputDir + "temp\\s905_geometry_normals_combined.img", 
				outputDir + "temp\\s905.hb",
				outputDir + "temp\\resynthesis_using_full.ply");
				*/
		
		//Image targetImage = new Image(outputDir + "s905_smooth\\s905.img"); 
		//LocalShortStats targetStats = new LocalShortStats(targetImage, 4);
		
		/*
		Mesh baseMesh = new Mesh(remeshedNoDisplacementDir + "tim.ply");
		Image startImage = new Image(displacementImageDir + "tim.img");
		startImage.addNoise();
		baseMesh.applyImageDetail(startImage);
		baseMesh.saveAs(outputDir + "s905_lowRes\\tim_with_noise.ply");
		*/
				
		
		LocalShortStats targetStats = new LocalShortStats(shortStatsDir + "s060.ss");
//		Image image =  new Image(displacementImageDir + "s905_lowRes4.img");
		Image image =  new Image(outputDir + "s905_s60\\s905_lowRes4_to_s056.img");
		image.matchStatisticsOf(targetStats);
		image.save(outputDir + "aging\\s905_lowRes4_to_s060.img");
		
		
		Mesh baseMesh = new Mesh(remeshedNoDisplacementDir + "s905_lowRes4.ply");
		baseMesh.applyImageDetail(image);
		baseMesh.saveAs(outputDir + "aging\\s905_lowRes4_to_s060.ply");
		
		
		/*
		adjustStats(remeshedNoDisplacementDir + "s905_lowRes.ply",
				displacementImageDir + "tim.img",
				targetStats, 
				outputDir + "s905_lowRes\\tim_to_s905.ply");
		*/
		
		/*
		LocalShortStats targetStats = new LocalShortStats(outputDir + "Interpolation\\meanStat.ss");
		adjustStats(outputDir + "Interpolation\\meanMesh_base.vec",
				outputDir + "Interpolation\\meanImage.img",
				targetStats, 
				outputDir + "Interpolation\\meanMeshAdjusted.ply");
				*/
		
		/*
		//targetImage = new Image(outputDir + "s905_smooth\\s905.img"); 
		//LocalShortStats targetStats = new LocalShortStats(targetImage, 4);
		LocalShortStats targetStats = new LocalShortStats(outputDir + "s905_smooth\\s905.ss");
		Image startImage = new Image(outputDir + "s905_smooth\\s60.img");
		startImage.matchStatisticsOf(targetStats);
		startImage.save(outputDir + "s905_smooth\\s60_to_s905_16tiles_4levels_noNoise_noMean.img");
		*/
		
		/*
		LocalICDF targetStats = new LocalICDF(targetImage, 6);
		adjustStats(outputDir + "s905_smooth\\s905_smooth_base.ply",
				outputDir + "s905_smooth\\s905_smooth.img",
				targetStats, 
				outputDir + "s905_smooth\\synthesis_6levels_full_hist_test.ply");
		*/
		
		/*
		Mesh baseMesh = new Mesh(outputDir + "s905_smooth\\s905_smooth_base.ply");
		baseMesh.applyImageDetail(outputDir + "s905_smooth\\s905_synthesized_32tiles.img");
		baseMesh.saveAs(outputDir + "s905_smooth\\synthesis_32tiles.ply");
		*/
	}
	
	public static Image targetImage = null;
	
	static void compareImages(Image img1, Image img2){
		/*
		Image img1 = new Image(imgFile1);
		Image img2 = new Image(imgFile2);
		*/
		int numLevels = 6;
		if(img1.pyramid == null)
			img1.createSteerablePyramid(4, numLevels);
		
		if(img2.pyramid == null)
			img2.createSteerablePyramid(4, numLevels);
		
		for(int i = 0; i < img1.pyramid.bands.length; i++){
			System.out.print("Channel: " + i + "; ");
			int numTiles = ImageLoader.numStatisticTiles;
			float totalError = 0;
			float totalAbsError = 0;
			for(int j = 0; j < numTiles; j++){
				for(int k = 0; k < numTiles; k++){
					float targetStd = img1.pyramid.bands[i].statistics[k][numTiles - j - 1].shortStats.std;
					float synthStd = img2.pyramid.bands[i].statistics[k][numTiles - j - 1].shortStats.std;
					float error = (synthStd - targetStd) / targetStd;
					totalError += error;
					totalAbsError += Math.abs(error);
					String errorStr = String.valueOf(error);
					
					if(errorStr.charAt(0) != '-')
						errorStr = "+" + errorStr;
					
					errorStr = errorStr.substring(0, 4);
					if(Math.abs(error) < .1)
						errorStr = "    ";
					else if(error < 0)
						errorStr = "-   ";
					else if(error > 0)
						errorStr = "+   ";
						
					//System.out.print(errorStr + "  ");
				}
				//System.out.println();
			}
			System.out.println("  mean error: " + totalError / numTiles / numTiles + 
					"; mean abs error: " + totalAbsError / numTiles / numTiles);
		}
	}
	
	static void lowResTrials(){
		
//		String toFiles[] = {"s031", "s035", "s036", "s037", "s038", "s040", "s044", "s046", "s049", "s055", "s056", "s065", "s071", "s073", "s074", "s076"};
		String toFiles[] = {"s036", "s073"};
		for(int k = 0; k < toFiles.length; k++){
			
			String toStr = "s060";
			for(int i = 13; i <= 13; i++){
				String startStr = "s905_lowRes" + i;
				/*
				System.gc();
				LocalShortStats targetStats = new LocalShortStats(shortStatsDir + toStr + ".ss");
				//Image image =  new Image(displacementImageDir + startStr + ".img");
				Image image =  new Image(outputDir + "lowResTest\\" + startStr + "_to_" + toFiles[k] + ".img");
				image.matchStatisticsOf(targetStats);
				image.save(outputDir + "lowResTest\\" + startStr + "_to_" + toFiles[k] + "_to_" + toStr + ".img");			
				
				Mesh baseMesh = new Mesh(remeshedNoDisplacementDir + startStr + ".ply");
				baseMesh.applyImageDetail(image);
				baseMesh.saveAs(outputDir + "lowResTest\\" + startStr + "_to_" + toFiles[k] + "_to_" + toStr + ".ply");
				*/
				adjustLowRes(startStr, "s060", outputDir + "lowResTest\\",
						outputDir + "lowResTest\\" + startStr + "_to_" + toFiles[k] + "_to_s060.img");
			}
			
		}
				
		
		//adjustLowRes("s905_lowRes10", "s060", outputDir + "lowResTest\\");
		//adjustLowRes("s905_lowRes11", "s060", outputDir + "lowResTest\\");
	}
	
	static void transferDetail(String from, String to, String workingDir){
		System.gc();
		Image image = new Image(displacementImageDir + from + ".img");

		
		Mesh baseMesh = new Mesh(baseMeshDir + to + ".ply");

		Mesh.details = new double[0][][]; 

		Mesh.readDetails(detailsDir + to + ".det");
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][0]);
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][1]);

		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][2]);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][3]);
		
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		
		baseMesh.applyImageDetail(image);
		baseMesh.saveAs(workingDir + from + "_to_" + to + ".ply");
	}

	static void age(String young, String old, String workingDir){
		System.gc();
		Mesh youngMesh = new Mesh(remeshedDir + young + ".ply");
		youngMesh.saveAs(workingDir + "comparison_young.ply");
		youngMesh = null;
		System.gc();
		
		Mesh baseMesh = new Mesh(baseMeshDir + young + ".ply");

		Mesh.details = new double[0][][]; 

		Mesh.readDetails(detailsDir + young + ".det");
		Mesh.readDetails(detailsDir + old + ".det");
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[0][0]);
		baseMesh.subdivide(Mesh.LINEAR_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][1]);

		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][2]);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.applyDetail(Mesh.details[1][3]);
		
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);
		baseMesh.subdivide(Mesh.LOOP_SUBDIVISION);

		Mesh noiseMesh = new Mesh(baseMesh);
		
		Image youngImage = new Image(displacementImageDir + young + ".img");
		baseMesh.applyImageDetail(youngImage);
		baseMesh.saveAs(workingDir + "comparison_young_warped.ply");
		baseMesh = null;
		System.gc();
		
		Mesh transferMesh = new Mesh(noiseMesh);

		Image youngNoiseImage = new Image(youngImage);
		youngNoiseImage.addNoise();
		noiseMesh.applyImageDetail(youngNoiseImage);
		noiseMesh.saveAs(workingDir + "comparison_perlin.ply");
		noiseMesh = null;
		System.gc();

		Mesh synthesizedMesh = new Mesh(transferMesh);
		
		Image oldImage = new Image(displacementImageDir + old + ".img");
		transferMesh.applyImageDetail(oldImage);
		transferMesh.saveAs(workingDir + "comparison_transferred.ply");
		transferMesh = null;
		System.gc();
		
		LocalShortStats oldStats = new LocalShortStats(shortStatsDir + old + ".ss"); 		
		youngImage.matchStatisticsOf(oldStats);		
		synthesizedMesh.applyImageDetail(youngImage);
		synthesizedMesh.saveAs(workingDir + "comparison_synthesized.ply");
		
		
		/*
		baseMesh.applyImageDetail(youngImage);
		baseMesh.saveAs(workingDir + FileUtils.getName(image.fileName) + "_det0111" + ".ply");
		*/
	}

	static void ageComparison(String ageComparisonDir){
		/*
		String[] youngFaces = {"s046", "s068-041112-02", 
				"s078-041118-02", "s089-041130-02", "s091-041130-02",
				"s108-041217-02", "s127", "s131", "s139"};
		*/
		String[] youngFaces = {"s089-041130-02", "s091-041130-02",
				"s108-041217-02", "s131", "s139"};

		
		//age("s127", "s060", ageComparisonDir + "s127\\");
		
		for(int i = 0; i < youngFaces.length; i++){
			try{
				String faceDirStr = ageComparisonDir + youngFaces[i] + "\\";
				File faceDir = new File(faceDirStr);
				if(!faceDir.exists())
					faceDir.mkdir();
				age(youngFaces[i], "s060_small", faceDirStr);
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	public static void rebuttal(){
		
		
		String rebuttalDir = outputDir + "rebuttal\\";
		ageComparison(rebuttalDir + "ageComparison\\");
		
		/*
		Mesh lowResBaseMesh = new Mesh(remeshedNoDisplacementDir + "s905_lowRes13.ply");
		Image lowResImage = new Image(displacementImageDir + "s905_lowRes13.img");
		lowResImage.addNoise();
		lowResBaseMesh.applyImageDetail(lowResImage);
		lowResBaseMesh.saveAs(rebuttalDir + "lowRes_PerlinNoise.ply");
		*/
		//mesh2bmp(rebuttalDir, 20, true);
		
		/*
		Image youngInitImage = new Image(rebuttalDir + "s133.img");
		youngInitImage.addNoise();
		youngInitImage.fileName = "PerlinNoise.img";
		
		//adjustLowRes("s133", "s060", rebuttalDir, youngInitImage);
		adjustLowRes("s133", "s060", rebuttalDir, rebuttalDir + "s060.img");
		*/
		/*
		Mesh youngBaseMesh = new Mesh(rebuttalDir + "s133_base.ply");

		
		youngBaseMesh.applyImageDetail(youngInitImage);
		
		youngBaseMesh.saveAs(rebuttalDir + "s133_PerlinNoise_over_1.ply");
		*/
		//transferDetail("s060", "s133", rebuttalDir);
		
	}
	
	static void presentation(){
		String interpolationDir = outputDir + "Interpolation\\";
		Mesh meanMeshBase = new Mesh(interpolationDir + "meanMesh_base.vec");
		meanMeshBase.saveAs("meanMesh_base.ply");
		
		Image meanImage = new Image(interpolationDir + "meanImage.img");
		LocalShortStats meanStats = new LocalShortStats(interpolationDir + "meanStat.ss");
		meanImage.matchStatisticsOf(meanStats);
		meanImage.save(interpolationDir + "meanImageAdjusted.img");
	}
	
	public static void main(String[] args) {
		//addFilesToInput();
		//alignMeshes();
		//createDetails();
		//createReMeshes();
		//createHBStats();
		
		//rebuttal();
		//presentation();
		Mesh.initFinalImageMesh();
		
		//testStatAdjustment();
		//aging();
		//lowResTrials();
		//removeFile("s060_small");
		/*
		compareImages(new Image(outputDir + "s905_smooth\\s905.img"), 
				new Image(outputDir + "s905_smooth\\s905_synthesized.img"));
		*/
		//img2ply(synthesisIterDir, remeshedDir + "s905_lowRes11.ply");
		
		//fixMarkerFile(markersDir + "s031.mkr", markersDir + "new_s031.mkr");
		//fixMarkerFiles();
		
		//vec2ply(eigenFaceDir, 5, 5);
		//processVectorDir(eigenFaceDir, Integer.MAX_VALUE, 0);
		//processVectorDir(projectionOnEigenFaceDir, 200, 5);
		//vec2ply(randomFaceDir);
		//processVectorDir(notInSetProjDir);
		//ply2vec(remeshedNoDisplacementDir);
		//vec2ply(outputDir + "baseDisplacement\\");
		//adjustStatsDir(eigenStatsDir, outputDir + "avgBase.vec", outputDir + "meanImage.img");
		//adjustStatsDir(randomShortStatsDir, outputDir + "avgBase.vec", outputDir + "meanImage.img");
		//adjustStatsDir(eigenShortStatsDir, outputDir + "avgBase.vec", outputDir + "meanImage.img");
		
		/*
		Image image = new Image(outputDir + "temp\\subject-30k.img"); 
		image.addNoise();
		Mesh mesh = new Mesh(outputDir + "subject-30k_base.ply");
		mesh.applyImageDetail(image);
		mesh.saveAs(outputDir + "temp\\meshNoise.ply");
		*/
		
		/*
		LocalICDF stats = new LocalICDF(new Image(outputDir + "temp\\s905_geometry_normals_combined.img"),
				4, ImageLoader.numStatisticTiles);
		stats.saveAs(outputDir + "temp\\s905.hb");
		*/
		
		//vec2ply(outputDir);
		//mesh2bmp(inputDir, 10, true);
		//mesh2bmp(remeshedDir, 1, false);
		//mesh2bmp(meshVectorDir + "WithHold\\", 6, false);
		//mesh2bmp(remeshedNoDisplacementDir, 1, false);
		//img2bmp(displacementImageDir, 5);
		
		/*
		String lowResFile = "subject-30k";
		adjustStats(remeshedNoDisplacementDir + lowResFile + ".ply",
				displacementImageDir + "WithHold\\" + lowResFile + ".img",
				outputDir + "meanStat.hb", outputDir + lowResFile + "Adjusted.ply");
		*/
		
		//testMakeDispImage();
		//testSaveHGStats();
		//testAdjustStats();
		//Mesh avgMesh = getAvgMesh((new File(remeshedNoDisplacementDir)).listFiles());
		//avgMesh.savePly(outputDir + "avgNoDisp.ply", true);
		
		//createDisplacementImages();
		/*
		Matrix4d trans = getRigidTrans("s60_geometry_normals_combined.mkr", 
			"s905_geometry_normals_combined.mkr");
		System.out.println(trans);
		
		Mesh fromMesh = new Mesh("s60_geometry_normals_combined.ply");
		Mesh toMesh = new Mesh("s905_geometry_normals_combined.ply");
		
		Mesh transformedMesh = new Mesh(fromMesh);
		Point3d pos = new Point3d(); 
		for(int i = 0; i < transformedMesh.vertices.size(); i++){
			Vertex v = (Vertex)transformedMesh.vertices.get(i);
			pos.set(v.pos);
			trans.transform(pos);
			v.pos.set(pos);
		}

		Vector3d center1 = new Vector3d(), center2 = new Vector3d();
		for(int i = 0; i < fromMesh.vertices.size(); i++){
			center1.add(((Vertex)fromMesh.vertices.get(i)).pos);
			center2.add(((Vertex)toMesh.vertices.get(i)).pos);
		}
		
		center1.scale(1.0 / fromMesh.vertices.size());
		center2.scale(1.0 / fromMesh.vertices.size());
		System.out.println("center 1: " + center1);
		System.out.println("center 2: " + center2);
		
		Vector3d diff = new Vector3d();
		diff.sub(center2, center1);
		System.out.println("difference: " + diff);
		//Loader loader = new Loader(new Mesh[]{new Mesh("s60_geometry_normals_combined.ply")});
		Loader loader = new Loader(new Mesh[]{fromMesh, transformedMesh, toMesh});
		loader.pack();
		loader.setVisible(true);
		*/
		System.exit(0);
	}

}
