/*
 * Created on Jul 22, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jogl;

import net.java.games.jogl.GL;

public class SimoncelliPyramid {
	int orientations = -1;
	int levels = -1;
	
	Image image;
	Image collapsedImage;
	Image difference;
	
	//filters
	Image filters[];
	
	//filtered images
	Image bands[];
	
	
	public SimoncelliPyramid(int orientations){ 
		if(orientations != 1 && orientations != 2 && orientations != 4)
			throw new RuntimeException("incorrect number of orientations: " + orientations);
		this.orientations = orientations; 
		loadFilters();
		//System.out.println("L0: " + L0Filter().sum() + "; L1: " + L1Filter().sum());
	}

	void loadFilters(){
		filters = new Image[5 + orientations];
		double filterArray[];
		switch(orientations){
			case 1:
				filterArray = Simoncelli_O1; break;
			case 2:
				filterArray = Simoncelli_O2; break;
			case 4:
				filterArray = Simoncelli_O4; break;
			default:
				throw new RuntimeException("wrong number of orientations");					
		}
		
		int numFilters = (int)Math.round(filterArray[1]);
		if(numFilters != filters.length)
			throw new RuntimeException("Number of filters not expected");
		
		int offset = 2;
		for(int i = 0; i < filters.length; i++){
			filters[i] = new Image(filterArray, offset);
			filters[i].createDrawnImage();
			offset += 2 + filters[i].width * filters[i].height;
			//System.out.println(filters[i]);
		}
	}
	
	void expand(Image I, int levels, EdgeHandler handler) 
	{
		System.out.println("  expanding pyramid; num levels: " + levels);
		if(levels <= 0)
			throw new RuntimeException("invalid number of pyramid levels");
		
		this.image = I;
		this.levels = levels;
		bands = new Image[(levels * orientations) + 2];
		
		// Apply the high and low-band filters
		bands[highPassIndex()] = new Image(I.width, I.height);
		Image.convolve( I, H0Filter(), highPass(), true, 1, 1, handler);
		bands[lowPassIndex()] = new Image(I.width, I.height);
		Image.convolve( I, L0Filter(), lowPass(), true, 1, 1, handler);

		// Generate the pyramid
		for( int iLevel = 0; iLevel < levels; iLevel++ )
		{
			// Generate the bandpass filter maps
			for( int iOrient = 0; iOrient < orientations; iOrient++ ){
				bands[bandIndex(iLevel, iOrient)] = new Image(lowPass().width, lowPass().height);
				Image.convolve( lowPass(), BandFilter(iOrient), band( iLevel, iOrient ), true, 1, 1, handler);
				/*
				System.out.println(iLevel + ", " + iOrient + ": " + bandIndex(iLevel, iOrient) + "| " + 
						band(iLevel, iOrient).width + "x" + band(iLevel, iOrient).height);
						*/				
			}

			// Generate the new lowpass filter map
			//cImage<X> NewLow;
			Image newLow = new Image(lowPass().width/2, lowPass().height/2);
			Image.convolve(lowPass(), L1Filter(), newLow, true, 2, 2, handler);
			bands[lowPassIndex()] = newLow;
			//System.out.println("    " + lowPass().min() + "/" + lowPass().max());
		}
	}

	Image collapse(EdgeHandler handler){
		System.out.println("  collapsing pyramid");
		Image I = new Image(lowPass());
		Image T = null;
		
		
		// Collapse the pyramid
		for( int iLevel = levels - 1; iLevel >= 0; iLevel--)
		{
			// Upsample the lowpass image
			//System.out.println("    " + I.min() + "/" + I.max());
			T = new Image(I.width * 2, I.height * 2);
			Image.upSample( I, T );
			I = new Image(T.width, T.height);
			Image.convolve( T, L1Filter(), I, false, 1, 1, handler);

			// Replace the bandpass portion at this level
			for( int iOrient = 0; iOrient < orientations; iOrient++ )
			{
				Image.convolve( band( iLevel, iOrient ), BandFilter(iOrient), T, false, 1, 1, handler);
				Image.add( I, T, I );
			}
		}
		
		// Undo the lowpass filtering
		Image.convolve( I, L0Filter(), T, false, 1, 1, handler);

		// Undo the highpass 
		Image.convolve( highPass(), H0Filter(), I, false, 1, 1, handler);
		Image.add( I, T, I );
		
		
		collapsedImage = I;
		difference = new Image(collapsedImage);
		float sum = 0;
		float maxError = 0;
		for(int i = 0; i < image.width; i++)
			for(int j = 0; j < image.height; j++){				
				float error = Math.abs(image.data[i][j] - collapsedImage.data[i][j]); 
				if(error > maxError)
					maxError = error;
				sum += error;
				difference.data[i][j] = error; 
			}
		System.out.println("    average error: " + sum/image.width/image.height + ", max error: " + maxError);
		return I;
	}

	
	Image highPass() { 
		return bands[0]; 
	}	
	
	// Access the lowpass filter map
	Image lowPass() { 
		return bands[1];
	}	

	// Access the bandpass images
	Image band( int level, int orient ) { 
		if(orient < 0 || orient >= orientations)
			throw new RuntimeException("Wrong orientation");
		return bands[ (level * orientations) + orient + 2 ]; 
	}

	int highPassIndex() { 
		return 0; 
	}	
	
	// Access the lowpass filter map
	int lowPassIndex() { 
		return 1;
	}	

	// Access the bandpass images
	int bandIndex( int level, int orient ) { 
		if(orient < 0 || orient >= orientations)
			throw new RuntimeException("Wrong orientation");
		return (level * orientations) + orient + 2; 
	}
	
	Image harmonics() { 
		return filters[0]; 
	}
	
	Image interp() { 
		return filters[1]; 
	}
	
	Image H0Filter() { 
		return filters[2]; 
	}
	Image L0Filter() { 
		return filters[3];
	}
	
	Image L1Filter() { 
		return filters[4]; 
	}
	
	Image BandFilter( int Orient ) { 
		return filters[ 5 + Orient ]; 
	}
	
	void draw(GL gl, int x, int y, boolean drawCDF){
		int myX = x;
		int myY = y;
		image.drawImage(gl, x, y, drawCDF);
		//highPass().drawImage(gl, x, y + image.height + 1);
		if(collapsedImage != null)
			collapsedImage.drawImage(gl, x, y + image.height + 1, drawCDF);
		/*
		difference.drawImage(gl, x, y + 2 * (image.height + 1));
		*/
		myX += highPass().width + 1;
		//System.out.println(bands.length);
		
		for(int i = 0; i < bands.length; i++){
			//System.out.println(bands[i]);
			//System.err.println(i + ": " + bands[i].width + "x" + bands[i].height);
		}
		
		for(int i = 0; i < levels; i++){
			for(int j = 0; j < orientations; j++){
				//System.out.println(i + ", " + j + ": " + bandIndex(i, j) + "| " + band(i, j).width + "x" + band(i, j).height);
				if(j%2 ==  0){
					myY = 0;
				}
				else{
					myY = band(i, j).height + 1;
				}
				band(i, j).drawImage(gl, myX, myY, drawCDF);
				if(j % 2 == 1){
					myX += band(i, j).width + 1;
				}
			}
		}
		
		lowPass().drawImage(gl, myX, y, drawCDF);
	}
	
	public void createHistograms(int bins, int numStatistics){
		image.createHistograms(bins, numStatistics);
		for(int i = 0; i < bands.length; i++)
			bands[i].createHistograms(bins, numStatistics);
	}	
	
	final static double Simoncelli_O1[] = {
            1,                   6,
            1,                   1,
            0,
            1,                   1,
            1,
            9,                   9,
   0.00059972,         -6.068e-005,         -0.00033249,         -0.00033256,         -0.00024066,
  -0.00033256,         -0.00033249,         -6.068e-005,          0.00059972,         -6.068e-005,          0.00012631,
   0.00049271,          0.00014597,         -0.00037321,          0.00014597,          0.00049271,          0.00012631,
  -6.068e-005,         -0.00033249,          0.00049271,         -0.00161665,          -0.0143736,          -0.0242014,
   -0.0143736,         -0.00161665,          0.00049271,         -0.00033249,         -0.00033256,          0.00014597,
   -0.0143736,          -0.0630092,          -0.0962359,          -0.0630092,          -0.0143736,          0.00014597,
  -0.00033256,         -0.00024066,         -0.00037321,          -0.0242014,          -0.0962359,            0.855489,
   -0.0962359,          -0.0242014,         -0.00037321,         -0.00024066,         -0.00033256,          0.00014597,
   -0.0143736,          -0.0630092,          -0.0962359,          -0.0630092,          -0.0143736,          0.00014597,
  -0.00033256,         -0.00033249,          0.00049271,         -0.00161665,          -0.0143736,          -0.0242014,
   -0.0143736,         -0.00161665,          0.00049271,         -0.00033249,         -6.068e-005,          0.00012631,
   0.00049271,          0.00014597,         -0.00037321,          0.00014597,          0.00049271,          0.00012631,
  -6.068e-005,          0.00059972,         -6.068e-005,         -0.00033249,         -0.00033256,         -0.00024066,
  -0.00033256,         -0.00033249,         -6.068e-005,          0.00059972,
            7,                   7,
   -0.0004514,         -0.00011371,         -0.00037258,         -0.00374386,         -0.00037258,
  -0.00011371,          -0.0004514,         -0.00011371,         -0.00611952,          -0.0134416,          -0.0075632,
   -0.0134416,         -0.00611952,         -0.00011371,         -0.00037258,          -0.0134416,           0.0644149,
     0.152494,           0.0644149,          -0.0134416,         -0.00037258,         -0.00374386,          -0.0075632,
     0.152494,            0.315302,            0.152494,          -0.0075632,         -0.00374386,         -0.00037258,
   -0.0134416,           0.0644149,            0.152494,           0.0644149,          -0.0134416,         -0.00037258,
  -0.00011371,         -0.00611952,          -0.0134416,          -0.0075632,          -0.0134416,         -0.00611952,
  -0.00011371,          -0.0004514,         -0.00011371,         -0.00037258,         -0.00374386,         -0.00037258,
  -0.00011371,          -0.0004514,
           13,                  13,
   -0.0002257,         -0.00080644,         -5.686e-005,          0.00087414,         -0.00018628,
  -0.00103164,         -0.00187192,         -0.00103164,         -0.00018628,          0.00087414,         -5.686e-005,
  -0.00080644,          -0.0002257,         -0.00080644,          0.00141762,         -0.00019038,         -0.00244906,
  -0.00459642,         -0.00700674,          -0.0069489,         -0.00700674,         -0.00459642,         -0.00244906,
  -0.00019038,          0.00141762,         -0.00080644,         -5.686e-005,         -0.00019038,         -0.00305976,
    -0.006401,          -0.0067208,         -0.00523618,          -0.0037816,         -0.00523618,          -0.0067208,
    -0.006401,         -0.00305976,         -0.00019038,         -5.686e-005,          0.00087414,         -0.00244906,
    -0.006401,         -0.00526002,          0.00393862,           0.0172208,            0.024496,           0.0172208,
   0.00393862,         -0.00526002,           -0.006401,         -0.00244906,          0.00087414,         -0.00018628,
  -0.00459642,          -0.0067208,          0.00393862,           0.0322074,           0.0630626,           0.0762467,
    0.0630626,           0.0322074,          0.00393862,          -0.0067208,         -0.00459642,         -0.00018628,
  -0.00103164,         -0.00700674,         -0.00523618,           0.0172208,           0.0630626,            0.111639,
       0.1349,            0.111639,           0.0630626,           0.0172208,         -0.00523618,         -0.00700674,
  -0.00103164,         -0.00187192,          -0.0069489,          -0.0037816,            0.024496,           0.0762467,
       0.1349,            0.157651,              0.1349,           0.0762467,            0.024496,          -0.0037816,
   -0.0069489,         -0.00187192,         -0.00103164,         -0.00700674,         -0.00523618,           0.0172208,
    0.0630626,            0.111639,              0.1349,            0.111639,           0.0630626,           0.0172208,
  -0.00523618,         -0.00700674,         -0.00103164,         -0.00018628,         -0.00459642,          -0.0067208,
   0.00393862,           0.0322074,           0.0630626,           0.0762467,           0.0630626,           0.0322074,
   0.00393862,          -0.0067208,         -0.00459642,         -0.00018628,          0.00087414,         -0.00244906,
    -0.006401,         -0.00526002,          0.00393862,           0.0172208,            0.024496,           0.0172208,
   0.00393862,         -0.00526002,           -0.006401,         -0.00244906,          0.00087414,         -5.686e-005,
  -0.00019038,         -0.00305976,           -0.006401,          -0.0067208,         -0.00523618,          -0.0037816,
  -0.00523618,          -0.0067208,           -0.006401,         -0.00305976,         -0.00019038,         -5.686e-005,
  -0.00080644,          0.00141762,         -0.00019038,         -0.00244906,         -0.00459642,         -0.00700674,
   -0.0069489,         -0.00700674,         -0.00459642,         -0.00244906,         -0.00019038,          0.00141762,
  -0.00080644,          -0.0002257,         -0.00080644,         -5.686e-005,          0.00087414,         -0.00018628,
  -0.00103164,         -0.00187192,         -0.00103164,         -0.00018628,          0.00087414,         -5.686e-005,
  -0.00080644,          -0.0002257,
            9,                   9,
  -9.066e-005,         -0.00173864,          -0.0049425,         -0.00788939,          -0.0100947,
  -0.00788939,          -0.0049425,         -0.00173864,         -9.066e-005,         -0.00173864,         -0.00462515,
  -0.00727254,         -0.00762341,         -0.00909195,         -0.00762341,         -0.00727254,         -0.00462515,
  -0.00173864,          -0.0049425,         -0.00727254,          -0.0212954,          -0.0243566,          -0.0348701,
   -0.0243566,          -0.0212954,         -0.00727254,          -0.0049425,         -0.00788939,         -0.00762341,
   -0.0243566,          -0.0173047,           -0.031586,          -0.0173047,          -0.0243566,         -0.00762341,
  -0.00788939,          -0.0100947,         -0.00909195,          -0.0348701,           -0.031586,            0.946419,
    -0.031586,          -0.0348701,         -0.00909195,          -0.0100947,         -0.00788939,         -0.00762341,
   -0.0243566,          -0.0173047,           -0.031586,          -0.0173047,          -0.0243566,         -0.00762341,
  -0.00788939,          -0.0049425,         -0.00727254,          -0.0212954,          -0.0243566,          -0.0348701,
   -0.0243566,          -0.0212954,         -0.00727254,          -0.0049425,         -0.00173864,         -0.00462515,
  -0.00727254,         -0.00762341,         -0.00909195,         -0.00762341,         -0.00727254,         -0.00462515,
  -0.00173864,         -9.066e-005,         -0.00173864,          -0.0049425,         -0.00788939,          -0.0100947,
  -0.00788939,          -0.0049425,         -0.00173864,         -9.066e-005
};


/////////////////////////////////////////////////
//Simoncelli Filters (2 orientations)


	final static double Simoncelli_O2[] = {
            1,                   7,
            1,                   1,
			   1,
            2,                   2,
            1,                   0,                   0,                   1,
            9,                   9,
    -0.000957,         -0.00024241,         -0.00142472,         -0.00087426,         -0.00116681,
  -0.00087426,         -0.00142472,         -0.00024241,           -0.000957,         -0.00024241,         -0.00431753,
   0.00089986,          0.00915642,           0.0109801,          0.00915642,          0.00089986,         -0.00431753,
  -0.00024241,         -0.00142472,          0.00089986,           0.0170635,           0.0109487,         -0.00589778,
    0.0109487,           0.0170635,          0.00089986,         -0.00142472,         -0.00087426,          0.00915642,
    0.0109487,          -0.0784137,           -0.156283,          -0.0784137,           0.0109487,          0.00915642,
  -0.00087426,         -0.00116681,           0.0109801,         -0.00589778,           -0.156283,            0.728259,
    -0.156283,         -0.00589778,           0.0109801,         -0.00116681,         -0.00087426,          0.00915642,
    0.0109487,          -0.0784137,           -0.156283,          -0.0784137,           0.0109487,          0.00915642,
  -0.00087426,         -0.00142472,          0.00089986,           0.0170635,           0.0109487,         -0.00589778,
    0.0109487,           0.0170635,          0.00089986,         -0.00142472,         -0.00024241,         -0.00431753,
   0.00089986,          0.00915642,           0.0109801,          0.00915642,          0.00089986,         -0.00431753,
  -0.00024241,           -0.000957,         -0.00024241,         -0.00142472,         -0.00087426,         -0.00116681,
  -0.00087426,         -0.00142472,         -0.00024241,           -0.000957,
            9,                   9,
  -8.701e-005,         -0.00135428,         -0.00160126,         -0.00050337,          0.00252401,
  -0.00050337,         -0.00160126,         -0.00135428,         -8.701e-005,         -0.00135428,          0.00292158,
   0.00752272,          0.00822442,          0.00110762,          0.00822442,          0.00752272,          0.00292158,
  -0.00135428,         -0.00160126,          0.00752272,         -0.00706129,          -0.0376949,          -0.0329714,
   -0.0376949,         -0.00706129,          0.00752272,         -0.00160126,         -0.00050337,          0.00822442,
   -0.0376949,           0.0438132,             0.18116,           0.0438132,          -0.0376949,          0.00822442,
  -0.00050337,          0.00252401,          0.00110762,          -0.0329714,             0.18116,            0.437625,
      0.18116,          -0.0329714,          0.00110762,          0.00252401,         -0.00050337,          0.00822442,
   -0.0376949,           0.0438132,             0.18116,           0.0438132,          -0.0376949,          0.00822442,
  -0.00050337,         -0.00160126,          0.00752272,         -0.00706129,          -0.0376949,          -0.0329714,
   -0.0376949,         -0.00706129,          0.00752272,         -0.00160126,         -0.00135428,          0.00292158,
   0.00752272,          0.00822442,          0.00110762,          0.00822442,          0.00752272,          0.00292158,
  -0.00135428,         -8.701e-005,         -0.00135428,         -0.00160126,         -0.00050337,          0.00252401,
  -0.00050337,         -0.00160126,         -0.00135428,         -8.701e-005,
           17,                  17,
   -4.35e-005,          0.00012078,         -0.00067714,         -0.00012434,         -0.00080064,
  -0.00159704,         -0.00025168,          -0.0004202,            0.001262,          -0.0004202,         -0.00025168,
  -0.00159704,         -0.00080064,         -0.00012434,         -0.00067714,          0.00012078,          -4.35e-005,
   0.00012078,          0.00044606,         -0.00058146,          0.00056216,         -0.00013688,          0.00232554,
   0.00288986,          0.00428728,           0.0055894,          0.00428728,          0.00288986,          0.00232554,
  -0.00013688,          0.00056216,         -0.00058146,          0.00044606,          0.00012078,         -0.00067714,
  -0.00058146,          0.00146078,          0.00216054,          0.00376136,          0.00308098,           0.0041122,
   0.00222122,          0.00055382,          0.00222122,           0.0041122,          0.00308098,          0.00376136,
   0.00216054,          0.00146078,         -0.00058146,         -0.00067714,         -0.00012434,          0.00056216,
   0.00216054,          0.00317578,          0.00318468,         -0.00177748,          -0.0074317,         -0.00905692,
  -0.00963722,         -0.00905692,          -0.0074317,         -0.00177748,          0.00318468,          0.00317578,
   0.00216054,          0.00056216,         -0.00012434,         -0.00080064,         -0.00013688,          0.00376136,
   0.00318468,         -0.00353064,          -0.0126042,          -0.0188474,          -0.0175082,          -0.0164857,
   -0.0175082,          -0.0188474,          -0.0126042,         -0.00353064,          0.00318468,          0.00376136,
  -0.00013688,         -0.00080064,         -0.00159704,          0.00232554,          0.00308098,         -0.00177748,
   -0.0126042,          -0.0202294,          -0.0110917,          0.00395566,           0.0143851,          0.00395566,
   -0.0110917,          -0.0202294,          -0.0126042,         -0.00177748,          0.00308098,          0.00232554,
  -0.00159704,         -0.00025168,          0.00288986,           0.0041122,          -0.0074317,          -0.0188474,
   -0.0110917,           0.0219066,           0.0680658,           0.0905801,           0.0680658,           0.0219066,
   -0.0110917,          -0.0188474,          -0.0074317,           0.0041122,          0.00288986,         -0.00025168,
   -0.0004202,          0.00428728,          0.00222122,         -0.00905692,          -0.0175082,          0.00395566,
    0.0680658,             0.14455,            0.177365,             0.14455,           0.0680658,          0.00395566,
   -0.0175082,         -0.00905692,          0.00222122,          0.00428728,          -0.0004202,            0.001262,
    0.0055894,          0.00055382,         -0.00963722,          -0.0164857,           0.0143851,           0.0905801,
     0.177365,            0.212037,            0.177365,           0.0905801,           0.0143851,          -0.0164857,
  -0.00963722,          0.00055382,           0.0055894,            0.001262,          -0.0004202,          0.00428728,
   0.00222122,         -0.00905692,          -0.0175082,          0.00395566,           0.0680658,             0.14455,
     0.177365,             0.14455,           0.0680658,          0.00395566,          -0.0175082,         -0.00905692,
   0.00222122,          0.00428728,          -0.0004202,         -0.00025168,          0.00288986,           0.0041122,
   -0.0074317,          -0.0188474,          -0.0110917,           0.0219066,           0.0680658,           0.0905801,
    0.0680658,           0.0219066,          -0.0110917,          -0.0188474,          -0.0074317,           0.0041122,
   0.00288986,         -0.00025168,         -0.00159704,          0.00232554,          0.00308098,         -0.00177748,
   -0.0126042,          -0.0202294,          -0.0110917,          0.00395566,           0.0143851,          0.00395566,
   -0.0110917,          -0.0202294,          -0.0126042,         -0.00177748,          0.00308098,          0.00232554,
  -0.00159704,         -0.00080064,         -0.00013688,          0.00376136,          0.00318468,         -0.00353064,
   -0.0126042,          -0.0188474,          -0.0175082,          -0.0164857,          -0.0175082,          -0.0188474,
   -0.0126042,         -0.00353064,          0.00318468,          0.00376136,         -0.00013688,         -0.00080064,
  -0.00012434,          0.00056216,          0.00216054,          0.00317578,          0.00318468,         -0.00177748,
   -0.0074317,         -0.00905692,         -0.00963722,         -0.00905692,          -0.0074317,         -0.00177748,
   0.00318468,          0.00317578,          0.00216054,          0.00056216,         -0.00012434,         -0.00067714,
  -0.00058146,          0.00146078,          0.00216054,          0.00376136,          0.00308098,           0.0041122,
   0.00222122,          0.00055382,          0.00222122,           0.0041122,          0.00308098,          0.00376136,
   0.00216054,          0.00146078,         -0.00058146,         -0.00067714,          0.00012078,          0.00044606,
  -0.00058146,          0.00056216,         -0.00013688,          0.00232554,          0.00288986,          0.00428728,
    0.0055894,          0.00428728,          0.00288986,          0.00232554,         -0.00013688,          0.00056216,
  -0.00058146,          0.00044606,          0.00012078,          -4.35e-005,          0.00012078,         -0.00067714,
  -0.00012434,         -0.00080064,         -0.00159704,         -0.00025168,          -0.0004202,            0.001262,
   -0.0004202,         -0.00025168,         -0.00159704,         -0.00080064,         -0.00012434,         -0.00067714,
   0.00012078,          -4.35e-005,
            9,                   9,
  -0.00612588,           0.0128742,          0.00564153,          0.00895726,                   0,
  -0.00895726,         -0.00564153,          -0.0128742,          0.00612588,           0.0080526,          0.00961152,
   -0.0041684,          0.00175117,                   0,         -0.00175117,           0.0041684,         -0.00961152,
   -0.0080526,           0.0210371,          -0.0102357,           0.0238218,           0.0183691,                   0,
   -0.0183691,          -0.0238218,           0.0102357,          -0.0210371,           0.0153689,         -0.00600945,
    0.0537532,           -0.126566,                   0,            0.126566,          -0.0537532,          0.00600945,
   -0.0153689,           0.0185147,         -0.00187262,           0.0207609,           -0.299617,                   0,
     0.299617,          -0.0207609,          0.00187262,          -0.0185147,           0.0153689,         -0.00600945,
    0.0537532,           -0.126566,                   0,            0.126566,          -0.0537532,          0.00600945,
   -0.0153689,           0.0210371,          -0.0102357,           0.0238218,           0.0183691,                   0,
   -0.0183691,          -0.0238218,           0.0102357,          -0.0210371,           0.0080526,          0.00961152,
   -0.0041684,          0.00175117,                   0,         -0.00175117,           0.0041684,         -0.00961152,
   -0.0080526,         -0.00612588,           0.0128742,          0.00564153,          0.00895726,                   0,
  -0.00895726,         -0.00564153,          -0.0128742,          0.00612588,
            9,                   9,
   0.00612588,          -0.0080526,          -0.0210371,          -0.0153689,          -0.0185147,
   -0.0153689,          -0.0210371,          -0.0080526,          0.00612588,          -0.0128742,         -0.00961152,
    0.0102357,          0.00600945,          0.00187262,          0.00600945,           0.0102357,         -0.00961152,
   -0.0128742,         -0.00564153,           0.0041684,          -0.0238218,          -0.0537532,          -0.0207609,
   -0.0537532,          -0.0238218,           0.0041684,         -0.00564153,         -0.00895726,         -0.00175117,
   -0.0183691,            0.126566,            0.299617,            0.126566,          -0.0183691,         -0.00175117,
  -0.00895726,                   0,                   0,                   0,                   0,                   0,
            0,                   0,                   0,                   0,          0.00895726,          0.00175117,
    0.0183691,           -0.126566,           -0.299617,           -0.126566,           0.0183691,          0.00175117,
   0.00895726,          0.00564153,          -0.0041684,           0.0238218,           0.0537532,           0.0207609,
    0.0537532,           0.0238218,          -0.0041684,          0.00564153,           0.0128742,          0.00961152,
   -0.0102357,         -0.00600945,         -0.00187262,         -0.00600945,          -0.0102357,          0.00961152,
    0.0128742,         -0.00612588,           0.0080526,           0.0210371,           0.0153689,           0.0185147,
    0.0153689,           0.0210371,           0.0080526,         -0.00612588
};


/////////////////////////////////////////////////
//Simoncelli Filters (4 orientations)


	final static double Simoncelli_O4[] = {
            1,                   9,
            1,                   2,
            1,                   3,
            4,                   4,
          0.5,              0.3536,                   0,             -0.3536,                   0,
       0.3536,                 0.5,              0.3536,                 0.5,             -0.3536,                   0,
       0.3536,                   0,              0.3536,                -0.5,              0.3536,
           15,                  15,
  -0.00040484,         -0.00062596,         -3.783e-005,          0.00088387,          0.00154508,
    0.0019236,          0.00206875,          0.00208987,          0.00206875,           0.0019236,          0.00154508,
   0.00088387,         -3.783e-005,         -0.00062596,         -0.00040484,         -0.00062596,         -0.00032735,
   0.00077435,          0.00158744,          0.00217507,          0.00256265,          0.00228922,          0.00197551,
   0.00228922,          0.00256265,          0.00217507,          0.00158744,          0.00077435,         -0.00032735,
  -0.00062596,         -3.783e-005,          0.00077435,          0.00117932,           0.0014051,          0.00222534,
   0.00211453,          0.00033578,         -0.00083369,          0.00033578,          0.00211453,          0.00222534,
    0.0014051,          0.00117932,          0.00077435,         -3.783e-005,          0.00088387,          0.00158744,
    0.0014051,           0.0012961,         -0.00049274,         -0.00312953,         -0.00457518,         -0.00510145,
  -0.00457518,         -0.00312953,         -0.00049274,           0.0012961,           0.0014051,          0.00158744,
   0.00088387,          0.00154508,          0.00217507,          0.00222534,         -0.00049274,         -0.00632227,
   -0.0027556,          0.00536322,          0.00730326,          0.00536322,          -0.0027556,         -0.00632227,
  -0.00049274,          0.00222534,          0.00217507,          0.00154508,           0.0019236,          0.00256265,
   0.00211453,         -0.00312953,          -0.0027556,           0.0139624,          0.00780463,         -0.00938122,
   0.00780463,           0.0139624,          -0.0027556,         -0.00312953,          0.00211453,          0.00256265,
    0.0019236,          0.00206875,          0.00228922,          0.00033578,         -0.00457518,          0.00536322,
   0.00780463,          -0.0795016,           -0.155414,          -0.0795016,          0.00780463,          0.00536322,
  -0.00457518,          0.00033578,          0.00228922,          0.00206875,          0.00208987,          0.00197551,
  -0.00083369,         -0.00510145,          0.00730326,         -0.00938122,           -0.155414,            0.730387,
    -0.155414,         -0.00938122,          0.00730326,         -0.00510145,         -0.00083369,          0.00197551,
   0.00208987,          0.00206875,          0.00228922,          0.00033578,         -0.00457518,          0.00536322,
   0.00780463,          -0.0795016,           -0.155414,          -0.0795016,          0.00780463,          0.00536322,
  -0.00457518,          0.00033578,          0.00228922,          0.00206875,           0.0019236,          0.00256265,
   0.00211453,         -0.00312953,          -0.0027556,           0.0139624,          0.00780463,         -0.00938122,
   0.00780463,           0.0139624,          -0.0027556,         -0.00312953,          0.00211453,          0.00256265,
    0.0019236,          0.00154508,          0.00217507,          0.00222534,         -0.00049274,         -0.00632227,
   -0.0027556,          0.00536322,          0.00730326,          0.00536322,          -0.0027556,         -0.00632227,
  -0.00049274,          0.00222534,          0.00217507,          0.00154508,          0.00088387,          0.00158744,
    0.0014051,           0.0012961,         -0.00049274,         -0.00312953,         -0.00457518,         -0.00510145,
  -0.00457518,         -0.00312953,         -0.00049274,           0.0012961,           0.0014051,          0.00158744,
   0.00088387,         -3.783e-005,          0.00077435,          0.00117932,           0.0014051,          0.00222534,
   0.00211453,          0.00033578,         -0.00083369,          0.00033578,          0.00211453,          0.00222534,
    0.0014051,          0.00117932,          0.00077435,         -3.783e-005,         -0.00062596,         -0.00032735,
   0.00077435,          0.00158744,          0.00217507,          0.00256265,          0.00228922,          0.00197551,
   0.00228922,          0.00256265,          0.00217507,          0.00158744,          0.00077435,         -0.00032735,
  -0.00062596,         -0.00040484,         -0.00062596,         -3.783e-005,          0.00088387,          0.00154508,
    0.0019236,          0.00206875,          0.00208987,          0.00206875,           0.0019236,          0.00154508,
   0.00088387,         -3.783e-005,         -0.00062596,         -0.00040484,
            9,                   9,
  -8.701e-005,         -0.00135428,         -0.00160126,         -0.00050337,          0.00252401,
  -0.00050337,         -0.00160126,         -0.00135428,         -8.701e-005,         -0.00135428,          0.00292158,
   0.00752272,          0.00822442,          0.00110762,          0.00822442,          0.00752272,          0.00292158,
  -0.00135428,         -0.00160126,          0.00752272,         -0.00706129,          -0.0376949,          -0.0329714,
   -0.0376949,         -0.00706129,          0.00752272,         -0.00160126,         -0.00050337,          0.00822442,
   -0.0376949,           0.0438132,             0.18116,           0.0438132,          -0.0376949,          0.00822442,
  -0.00050337,          0.00252401,          0.00110762,          -0.0329714,             0.18116,            0.437625,
      0.18116,          -0.0329714,          0.00110762,          0.00252401,         -0.00050337,          0.00822442,
   -0.0376949,           0.0438132,             0.18116,           0.0438132,          -0.0376949,          0.00822442,
  -0.00050337,         -0.00160126,          0.00752272,         -0.00706129,          -0.0376949,          -0.0329714,
   -0.0376949,         -0.00706129,          0.00752272,         -0.00160126,         -0.00135428,          0.00292158,
   0.00752272,          0.00822442,          0.00110762,          0.00822442,          0.00752272,          0.00292158,
  -0.00135428,         -8.701e-005,         -0.00135428,         -0.00160126,         -0.00050337,          0.00252401,
  -0.00050337,         -0.00160126,         -0.00135428,         -8.701e-005,
           17,                  17,
   -4.35e-005,          0.00012078,         -0.00067714,         -0.00012434,         -0.00080064,
  -0.00159704,         -0.00025168,          -0.0004202,            0.001262,          -0.0004202,         -0.00025168,
  -0.00159704,         -0.00080064,         -0.00012434,         -0.00067714,          0.00012078,          -4.35e-005,
   0.00012078,          0.00044606,         -0.00058146,          0.00056216,         -0.00013688,          0.00232554,
   0.00288986,          0.00428728,           0.0055894,          0.00428728,          0.00288986,          0.00232554,
  -0.00013688,          0.00056216,         -0.00058146,          0.00044606,          0.00012078,         -0.00067714,
  -0.00058146,          0.00146078,          0.00216054,          0.00376136,          0.00308098,           0.0041122,
   0.00222122,          0.00055382,          0.00222122,           0.0041122,          0.00308098,          0.00376136,
   0.00216054,          0.00146078,         -0.00058146,         -0.00067714,         -0.00012434,          0.00056216,
   0.00216054,          0.00317578,          0.00318468,         -0.00177748,          -0.0074317,         -0.00905692,
  -0.00963722,         -0.00905692,          -0.0074317,         -0.00177748,          0.00318468,          0.00317578,
   0.00216054,          0.00056216,         -0.00012434,         -0.00080064,         -0.00013688,          0.00376136,
   0.00318468,         -0.00353064,          -0.0126042,          -0.0188474,          -0.0175082,          -0.0164857,
   -0.0175082,          -0.0188474,          -0.0126042,         -0.00353064,          0.00318468,          0.00376136,
  -0.00013688,         -0.00080064,         -0.00159704,          0.00232554,          0.00308098,         -0.00177748,
   -0.0126042,          -0.0202294,          -0.0110917,          0.00395566,           0.0143851,          0.00395566,
   -0.0110917,          -0.0202294,          -0.0126042,         -0.00177748,          0.00308098,          0.00232554,
  -0.00159704,         -0.00025168,          0.00288986,           0.0041122,          -0.0074317,          -0.0188474,
   -0.0110917,           0.0219066,           0.0680658,           0.0905801,           0.0680658,           0.0219066,
   -0.0110917,          -0.0188474,          -0.0074317,           0.0041122,          0.00288986,         -0.00025168,
   -0.0004202,          0.00428728,          0.00222122,         -0.00905692,          -0.0175082,          0.00395566,
    0.0680658,             0.14455,            0.177365,             0.14455,           0.0680658,          0.00395566,
   -0.0175082,         -0.00905692,          0.00222122,          0.00428728,          -0.0004202,            0.001262,
    0.0055894,          0.00055382,         -0.00963722,          -0.0164857,           0.0143851,           0.0905801,
     0.177365,            0.212037,            0.177365,           0.0905801,           0.0143851,          -0.0164857,
  -0.00963722,          0.00055382,           0.0055894,            0.001262,          -0.0004202,          0.00428728,
   0.00222122,         -0.00905692,          -0.0175082,          0.00395566,           0.0680658,             0.14455,
     0.177365,             0.14455,           0.0680658,          0.00395566,          -0.0175082,         -0.00905692,
   0.00222122,          0.00428728,          -0.0004202,         -0.00025168,          0.00288986,           0.0041122,
   -0.0074317,          -0.0188474,          -0.0110917,           0.0219066,           0.0680658,           0.0905801,
    0.0680658,           0.0219066,          -0.0110917,          -0.0188474,          -0.0074317,           0.0041122,
   0.00288986,         -0.00025168,         -0.00159704,          0.00232554,          0.00308098,         -0.00177748,
   -0.0126042,          -0.0202294,          -0.0110917,          0.00395566,           0.0143851,          0.00395566,
   -0.0110917,          -0.0202294,          -0.0126042,         -0.00177748,          0.00308098,          0.00232554,
  -0.00159704,         -0.00080064,         -0.00013688,          0.00376136,          0.00318468,         -0.00353064,
   -0.0126042,          -0.0188474,          -0.0175082,          -0.0164857,          -0.0175082,          -0.0188474,
   -0.0126042,         -0.00353064,          0.00318468,          0.00376136,         -0.00013688,         -0.00080064,
  -0.00012434,          0.00056216,          0.00216054,          0.00317578,          0.00318468,         -0.00177748,
   -0.0074317,         -0.00905692,         -0.00963722,         -0.00905692,          -0.0074317,         -0.00177748,
   0.00318468,          0.00317578,          0.00216054,          0.00056216,         -0.00012434,         -0.00067714,
  -0.00058146,          0.00146078,          0.00216054,          0.00376136,          0.00308098,           0.0041122,
   0.00222122,          0.00055382,          0.00222122,           0.0041122,          0.00308098,          0.00376136,
   0.00216054,          0.00146078,         -0.00058146,         -0.00067714,          0.00012078,          0.00044606,
  -0.00058146,          0.00056216,         -0.00013688,          0.00232554,          0.00288986,          0.00428728,
    0.0055894,          0.00428728,          0.00288986,          0.00232554,         -0.00013688,          0.00056216,
  -0.00058146,          0.00044606,          0.00012078,          -4.35e-005,          0.00012078,         -0.00067714,
  -0.00012434,         -0.00080064,         -0.00159704,         -0.00025168,          -0.0004202,            0.001262,
   -0.0004202,         -0.00025168,         -0.00159704,         -0.00080064,         -0.00012434,         -0.00067714,
   0.00012078,          -4.35e-005,
            9,                   9,
  -0.00081125,          0.00391035,          0.00134627,          0.00074701,                   0,
  -0.00074701,         -0.00134627,         -0.00391035,          0.00081125,          0.00444516,          0.00445654,
  -0.00377407,         -0.00036522,                   0,          0.00036522,          0.00377407,         -0.00445654,
  -0.00444516,            0.012317,         -0.00587243,          0.00825816,          -0.0225227,                   0,
    0.0225227,         -0.00825816,          0.00587243,           -0.012317,           0.0139559,         -0.00287608,
    0.0394423,           -0.110569,                   0,            0.110569,          -0.0394423,          0.00287608,
   -0.0139559,           0.0141795,          0.00852676,           0.0536056,           -0.176842,                   0,
     0.176842,          -0.0536056,         -0.00852676,          -0.0141795,           0.0139559,         -0.00287608,
    0.0394423,           -0.110569,                   0,            0.110569,          -0.0394423,          0.00287608,
   -0.0139559,            0.012317,         -0.00587243,          0.00825816,          -0.0225227,                   0,
    0.0225227,         -0.00825816,          0.00587243,           -0.012317,          0.00444516,          0.00445654,
  -0.00377407,         -0.00036522,                   0,          0.00036522,          0.00377407,         -0.00445654,
  -0.00444516,         -0.00081125,          0.00391035,          0.00134627,          0.00074701,                   0,
  -0.00074701,         -0.00134627,         -0.00391035,          0.00081125,
            9,                   9,
  -0.00312215,          0.00862044,           0.0148076,          0.00808719,          0.00466709,
   4.011e-005,         -5.711e-005,         -0.00082847,                   0,          0.00862044,           0.0149175,
  -0.00448149,          0.00232988,         -0.00208656,          -0.0069719,          -0.0009748,                   0,
   0.00082847,           0.0148076,         -0.00448149,           0.0327859,           0.0507971,          -0.0244273,
   -0.0121455,                   0,           0.0009748,          5.711e-005,          0.00808719,          0.00232988,
    0.0507971,          -0.0824955,           -0.151056,                   0,           0.0121455,           0.0069719,
  -4.011e-005,          0.00466709,         -0.00208656,          -0.0244273,           -0.151056,                   0,
     0.151056,           0.0244273,          0.00208656,         -0.00466709,          4.011e-005,          -0.0069719,
   -0.0121455,                   0,            0.151056,           0.0824955,          -0.0507971,         -0.00232988,
  -0.00808719,         -5.711e-005,          -0.0009748,                   0,           0.0121455,           0.0244273,
   -0.0507971,          -0.0327859,          0.00448149,          -0.0148076,         -0.00082847,                   0,
    0.0009748,           0.0069719,          0.00208656,         -0.00232988,          0.00448149,          -0.0149175,
  -0.00862044,                   0,          0.00082847,          5.711e-005,         -4.011e-005,         -0.00466709,
  -0.00808719,          -0.0148076,         -0.00862044,          0.00312215,
            9,                   9,
  -0.00081125,          0.00444516,            0.012317,           0.0139559,           0.0141795,
    0.0139559,            0.012317,          0.00444516,         -0.00081125,          0.00391035,          0.00445654,
  -0.00587243,         -0.00287608,          0.00852676,         -0.00287608,         -0.00587243,          0.00445654,
   0.00391035,          0.00134627,         -0.00377407,          0.00825816,           0.0394423,           0.0536056,
    0.0394423,          0.00825816,         -0.00377407,          0.00134627,          0.00074701,         -0.00036522,
   -0.0225227,           -0.110569,           -0.176842,           -0.110569,          -0.0225227,         -0.00036522,
   0.00074701,                   0,                   0,                   0,                   0,                   0,
            0,                   0,                   0,                   0,         -0.00074701,          0.00036522,
    0.0225227,            0.110569,            0.176842,            0.110569,           0.0225227,          0.00036522,
  -0.00074701,         -0.00134627,          0.00377407,         -0.00825816,          -0.0394423,          -0.0536056,
   -0.0394423,         -0.00825816,          0.00377407,         -0.00134627,         -0.00391035,         -0.00445654,
   0.00587243,          0.00287608,         -0.00852676,          0.00287608,          0.00587243,         -0.00445654,
  -0.00391035,          0.00081125,         -0.00444516,           -0.012317,          -0.0139559,          -0.0141795,
   -0.0139559,           -0.012317,         -0.00444516,          0.00081125,
            9,                   9,
            0,         -0.00082847,         -5.711e-005,          4.011e-005,          0.00466709,
   0.00808719,           0.0148076,          0.00862044,         -0.00312215,          0.00082847,                   0,
   -0.0009748,          -0.0069719,         -0.00208656,          0.00232988,         -0.00448149,           0.0149175,
   0.00862044,          5.711e-005,           0.0009748,                   0,          -0.0121455,          -0.0244273,
    0.0507971,           0.0327859,         -0.00448149,           0.0148076,         -4.011e-005,           0.0069719,
    0.0121455,                   0,           -0.151056,          -0.0824955,           0.0507971,          0.00232988,
   0.00808719,         -0.00466709,          0.00208656,           0.0244273,            0.151056,                   0,
    -0.151056,          -0.0244273,         -0.00208656,          0.00466709,         -0.00808719,         -0.00232988,
   -0.0507971,           0.0824955,            0.151056,                   0,          -0.0121455,          -0.0069719,
   4.011e-005,          -0.0148076,          0.00448149,          -0.0327859,          -0.0507971,           0.0244273,
    0.0121455,                   0,          -0.0009748,         -5.711e-005,         -0.00862044,          -0.0149175,
   0.00448149,         -0.00232988,          0.00208656,           0.0069719,           0.0009748,                   0,
  -0.00082847,          0.00312215,         -0.00862044,          -0.0148076,         -0.00808719,         -0.00466709,
  -4.011e-005,          5.711e-005,          0.00082847,                   0
};

}
