package jogl;

import java.util.*;

public class WrinkleStats {
	WrinkleCluster[] wrinkleClusters;
	
	public WrinkleStats(Image[] images){		
		List wrinkles = new ArrayList();
		
		int numClusters = images[0].width / 512 * 20;
		WrinkleVector.numClusters = numClusters;
		
		for(int i = 0; i < images.length; i++){
			wrinkles.addAll(Wrinkle.getWrinkles(images[i]));
			images[i].wrinkleVector = new WrinkleVector();
		}
		
		wrinkleClusters = WrinkleModel.getWrinkleClusters(wrinkles, numClusters);
		if(wrinkleClusters.length != numClusters)
			throw new RuntimeException("Something wrong");
		
		for(int i = 0 ; i < numClusters; i++){
			for(int j = 0; j < wrinkleClusters[i].wrinkles.length; j++){
				wrinkleClusters[i].wrinkles[j].originalWrinkle.sourceImg.wrinkleVector.vector[i]++;				
			}
		}

		for(int i = 0; i < numClusters; i++){
			System.out.print(i + ": ");
			for(int j = 0; j < images.length; j++){
				System.out.print(images[j].wrinkleVector.vector[i] + "\t");
			}
			System.out.print("\n");
		}
	}
	
	public static Image adjustImage(Image image, WrinkleVector targetVector, WrinkleCluster clusters[]){
		if(targetVector.vector.length != clusters.length)
			throw new RuntimeException();
		
		if(image.wrinkleVector == null)
			image.wrinkleVector = new WrinkleVector();
		
		List wrinklesToAdd = new ArrayList();
		
		boolean chosenWrinkles[][] = new boolean[clusters.length][];
		for(int i = 0; i < clusters.length; i++){
			chosenWrinkles[i] = new boolean[clusters[i].wrinkles.length];
			for(int j = 0; j < chosenWrinkles[i].length; j++)
				chosenWrinkles[i][j] = false;
		}
		
		Random rand = new Random();
		System.out.println("  adding wrinkles");
		for(int i = 0; i < clusters.length; i++){
			System.out.print("\n\n  " + i + ": ");
			while(image.wrinkleVector.vector[i] < targetVector.vector[i]){
				
				int wrinkleIndex = rand.nextInt(clusters[i].wrinkles.length);
				while(chosenWrinkles[i][wrinkleIndex])
					wrinkleIndex = rand.nextInt(clusters[i].wrinkles.length);
				chosenWrinkles[i][wrinkleIndex] = true;
				wrinklesToAdd.add(clusters[i].wrinkles[wrinkleIndex].originalWrinkle);				
				System.out.print(wrinkleIndex + "   ");
				image.wrinkleVector.vector[i]++;
			}
		}
		
		System.out.println("  getting wrinkle patches");
		List wrinklePatches = new ArrayList();
		for(int i = 0; i < wrinklesToAdd.size(); i++){
			if(i % 100 == 0)
				System.out.println("    " + i + " / " + wrinklesToAdd.size());
			Wrinkle wrinkle = (Wrinkle)wrinklesToAdd.get(i);
			wrinklePatches.add(new WrinklePatch(wrinkle, wrinkle.sourceImg));
		}
		
		Image wrinkleImage = new Image(image.width, image.height);
		wrinkleImage.applyWrinklePatches(wrinklePatches);
		image.add(wrinkleImage);
		return wrinkleImage;
	}
	
}
